using System;
using System.IO;
using System.Xml;
using System.Windows.Forms;

namespace Geocode
{
	public class NLGeocode
	{
		public static int ResponseSize(string searchString)
		{
			int y;
			y = 0;
			
			XmlReader reader = XmlReader.Create(searchString);
			
			while (reader.Read())
			{
				if (reader.NodeType == XmlNodeType.Element && reader.Name == "xls:GeocodeResponseList")
				{
					y = Convert.ToInt32(reader.GetAttribute(0));
				}
			}
			reader.Close();
			return y;
		}
		
		public static void NLGeocoder(string searchString, string[] resultString)
		{
			string street, buildingNumber, postalCode, town, municipality, province, xCoordinate, yCoordinate;
			int x, y, resultLevel;
			resultLevel = 0;
			x = y = -1;
			street = buildingNumber = postalCode = town = municipality = province = xCoordinate = yCoordinate = null;
			
			//XmlReader Class represents a reader that provides fast, noncached, forward-only access to XML data.
			XmlReader reader = XmlReader.Create(searchString);
			
			while (reader.Read())
			{
				switch (reader.Name)
				{
					case "xls:GeocodeResponseList":
						if (reader.NodeType == XmlNodeType.Element)
						{
							y = Convert.ToInt32(reader.GetAttribute(0));
							Array.Resize(ref resultString, y);
						}
						break;
					case "xls:GeocodedAddress":
						if (reader.NodeType == XmlNodeType.Element)
						{
							if (x != -1)
							{
							resultString[x] = resultLevel + "," + street + "," + buildingNumber + "," + postalCode + "," + town + "," + municipality + "," + province + "," + xCoordinate + "," + yCoordinate;
							}
							x++;
							resultLevel = 0;
						}
						break;
					case "gml:pos":
						if (reader.NodeType == XmlNodeType.Element)
						{
							reader.Read();
							string[] coords = reader.Value.Split(new char[] { ' ' });
							xCoordinate = coords[0];
							yCoordinate = coords[1];
						}
						break;
					case "xls:Building":
						if (reader.HasAttributes)
						{
							buildingNumber = "";
							while (reader.MoveToNextAttribute())
							{
								if (buildingNumber == "")
								{
									buildingNumber = reader.Value;
								}
								else
								{
									buildingNumber = buildingNumber + " " + reader.Value;
								}
							}
							resultLevel = 1;
						}
						break;
					case "xls:Street":
						if (reader.NodeType == XmlNodeType.Element)
						{
							reader.Read();
							street = reader.Value;
							if (resultLevel == 0)
							{
								resultLevel = 3;
							}
						}
						break;
					case "xls:Place":
						if (reader.NodeType == XmlNodeType.Element && reader.GetAttribute(0) == "MunicipalitySubdivision")
						{
							reader.Read();
							town = reader.Value;
							if (resultLevel == 0)
							{
								resultLevel = 4;
							}
						}
						else if (reader.NodeType == XmlNodeType.Element  && reader.GetAttribute(0) == "Municipality")
						{
							reader.Read();
							municipality = reader.Value;
							if (resultLevel == 0)
							{
								resultLevel = 5;
							}
						}
						else if (reader.NodeType == XmlNodeType.Element && reader.GetAttribute(0) == "CountrySubdivision")
						{
							reader.Read();
							province = reader.Value;
							if (resultLevel == 0)
							{
								resultLevel = 6;
							}
						}
						break;
					case "xls:PostalCode":
						if (reader.NodeType == XmlNodeType.Element)
						{
							reader.Read();
							postalCode = reader.Value;
							if (resultLevel > 1)
							{
								resultLevel = 2;
							}
						}
						break;
				} 
			}
			resultString[x] = resultLevel + "," + street + "," + buildingNumber + "," + postalCode + "," + town + "," + municipality + "," + province + "," + xCoordinate + "," + yCoordinate;
			reader.Close();
		}
	}
}
// The DIALOGSLib Namespace
namespace DIALOGSLib
{
   // The NewDialogs class
   class NewDialogs
	{
		public static void DLGNote(string sNotification, string sCaption)
		{
		string notification = sNotification;
		string caption = sCaption;
		MessageBox.Show(notification, caption, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
		}
	  public static bool DLGAsk(string sQuestion, string sCaption)
		{
		string question = sQuestion;
		string caption = sCaption;
		var result = MessageBox.Show(question, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
			if (result == DialogResult.Yes)
				{
					return true;
				}
			else
				{
					return false;
				}
		}
	  public static bool DLGWarn(string sWarning, string sCaption)
		{
		string warning = sWarning;
		string caption = sCaption;
		var result = MessageBox.Show(warning, caption, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
			if (result == DialogResult.OK)
				{
					return true;
				}
			else
				{
					return false;
				}
		}
		public static string BrowseForFolder(string sDescription, string sFolder)
			{
				FolderBrowserDialog dlg = new FolderBrowserDialog();
				dlg.SelectedPath = sFolder;
				dlg.Description = sDescription;
				dlg.ShowNewFolderButton = true;
				if (dlg.ShowDialog() == DialogResult.OK)
					return dlg.SelectedPath;
				else
					return "";
			}
	}
}
// The MBExtensions Namespace
namespace MBExtensions
{
 	// The MBFileManagement class
	class MBFileManagement
	{
		public static void OpenFileOrDir(string sOpen)
			{
				System.Diagnostics.Process.Start(sOpen);
			}
	}
}
// The DATETIMELib Namespace
namespace DATETIMELib
{
   // The MBDateAndTime class
   class MBDateAndTime
   {
      public static string RegionalLongDate(string sDateString)
      {
        string mbDateString;
        DateTime stringToDate;
 
        stringToDate = ConvertMBDate(sDateString);
 
        /* Convert the DateTime object stringToDate to
        it's equivalent long date string representation
        (depends on the Regional and Language Options) */
        mbDateString = (stringToDate.ToLongDateString());
         
        return mbDateString;
      }
       
      public static DateTime ConvertMBDate(string sDateString)
      /* The input string sDateString which comes from your
      MapBasic app will be either a Date (i.e. YYYYMMDD) or
      a DateTime (i.e. YYYYMMDDHHMMSSFFF) */
      {
          int mbYear;
          int mbMonth;
          int mbDay;
          DateTime stringToDate;
 
          /* Take the Year component from the input string
          (i.e. YYYY) and convert it to an integer */
          mbYear = Convert.ToInt32(sDateString.Substring(0, 4));
 
          /* Take the Month component from the input string
          (i.e. MM) and convert it to an integer */
          mbMonth = Convert.ToInt32(sDateString.Substring(4, 2));
 
          /* Take the Day component from the input string
          (i.e. DD) and convert it to an integer */
          mbDay = Convert.ToInt32(sDateString.Substring(6, 2));
 
          // Assign a value to the DateTime object stringToDate
          stringToDate = new DateTime(mbYear, mbMonth, mbDay);
 
          return stringToDate;
      }
  }
}
